# LLNG wrapper class to enable ServiceToken handler with FastCGI handler
#
# See http://lemonldap-ng.org/documentation/latest/handlerarch
package Lemonldap::NG::Handler::PSGI::ServiceToken;

use strict;

use base 'Lemonldap::NG::Handler::Lib::ServiceToken',
  'Lemonldap::NG::Handler::PSGI::Main';

our $VERSION = '1.9.991_01';

1;
